@@ -1,7 +1,7 @@
 //! Helpers for code generation that don't need macro expansion.
 
-use ir::context::BindgenContext;
-use ir::layout::Layout;
+use crate::ir::context::BindgenContext;
+use crate::ir::layout::Layout;
 use proc_macro2::{Ident, Span, TokenStream};
 use quote::TokenStreamExt;
 
@@ -136,13 +136,34 @@ pub fn bitfield_unit(ctx: &BindgenContext, layout: Layout) -> TokenStream {
 }
 
 pub mod ast_ty {
-    use ir::context::BindgenContext;
-    use ir::function::FunctionSig;
-    use ir::layout::Layout;
-    use ir::ty::FloatKind;
+    use crate::ir::context::BindgenContext;
+    use crate::ir::function::FunctionSig;
+    use crate::ir::layout::Layout;
+    use crate::ir::ty::FloatKind;
     use proc_macro2::{self, TokenStream};
     use std::str::FromStr;
 
+    pub fn c_void(ctx: &BindgenContext) -> TokenStream {
+        // ctypes_prefix takes precedence
+        match ctx.options().ctypes_prefix {
+            Some(ref prefix) => {
+                let prefix = TokenStream::from_str(prefix.as_str()).unwrap();
+                quote! {
+                    #prefix::c_void
+                }
+            }
+            None => {
+                if ctx.options().use_core &&
+                    ctx.options().rust_features.core_ffi_c_void
+                {
+                    quote! { ::core::ffi::c_void }
+                } else {
+                    quote! { ::std::os::raw::c_void }
+                }
+            }
+        }
+    }
+
     pub fn raw_type(ctx: &BindgenContext, name: &str) -> TokenStream {
         let ident = ctx.rust_ident_raw(name);
         match ctx.options().ctypes_prefix {
